"""Qt6 based GUI frontend for mozilla VPN."""
import csv
import subprocess

from PyQt6.QtWidgets import QApplication, QLabel, QWidget, QVBoxLayout, QHBoxLayout, \
    QPushButton, QComboBox, QMessageBox, QSizePolicy, QMainWindow


class MainWindow(QMainWindow):
    """
    MozVPN GUI window.
    """
    def __init__(self):
        """Compose main window widgets."""
        super().__init__()
        self.vpn_active = False

        self.setWindowTitle('MozVPN')

        vlayout = QVBoxLayout()
        vlayout.addWidget(QLabel('<h2>MozillaVPN Connector</h2>'))

        self.combo = QComboBox()
        self._fill_vpn_choice_combo()
        vlayout.addWidget(self.combo)

        self.connect_button = QPushButton('Connect')
        self.connect_button.clicked.connect(self.toggle_connect)
        self.connect_button.setSizePolicy(
            QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Minimum
        )
        hlayout = QHBoxLayout()
        hlayout.addStretch()
        hlayout.addWidget(self.connect_button)
        hlayout.addStretch()
        vlayout.addLayout(hlayout)

        container = QWidget()
        container.setLayout(vlayout)
        self.setCentralWidget(container)

    def _fill_vpn_choice_combo(self):
        """Fill the VPN server combobox from geolocation csv file."""
        fp = open('server-locations.csv')
        server_locations = csv.DictReader(fp)
        for loc in server_locations:
            loc_str = '{country} - {region} - {city}'.format(**loc)
            self.combo.addItem(loc_str, loc['conf'])

    def closeEvent(self, event):
        """Catch close event. If active VPN connection, ask for confirmation."""
        if self.vpn_active:
            reply = QMessageBox.question(
                self, 'Quit', 'Are you sure you want to quit and stop the active VPN?'
            )
            if reply == QMessageBox.StandardButtons.Yes:
                self.run_wg_quick_cmd('down')
            else:
                event.ignore()

    def toggle_connect(self):
        """Toggle (connect or disconnect) Mozilla VPN connection."""
        if self.vpn_active:
            button_text = 'Connect'
            button_color = 'none'
            vpn_cmd_arg = 'down'
        else:
            button_text = 'Disconnect'
            button_color = 'red'
            vpn_cmd_arg = 'up'

        if not self.run_wg_quick_cmd(vpn_cmd_arg):
            # Running the VPN command was not successful!
            return

        self.vpn_active = not self.vpn_active
        self.connect_button.setText(button_text)
        self.connect_button.setStyleSheet(f"background-color: {button_color}")
        self.combo.setEnabled(not self.vpn_active)

    def run_wg_quick_cmd(self, vpn_cmd_arg: str):
        """Run the external 'sudo wg-quick <up/down> cfg-file command.

        Args:
            vpn_cmd_arg: either 'up' or 'down'
        Returns:
            True if successful, else False
        """
        vpn_config_file = self.combo.currentData()
        wg_quick_cmd = f'xecho sudo -n wg-quick {vpn_cmd_arg} {vpn_config_file}'.split()
        try:
            proc = subprocess.run(wg_quick_cmd, timeout=10, capture_output=True)
        except FileNotFoundError as exc:
            QMessageBox.critical(self, 'Error', str(exc))
            return False
        if proc.returncode:
            QMessageBox.critical(self, 'Error', proc.stderr.decode('utf8'))
            return False
        return True


def gui():
    """Main gui function, to be called from cli.py"""
    app = QApplication([])
    window = MainWindow()
    window.show()
    app.exec()



"""Main module."""
import re
import pathlib
import logging
from typing import List, Dict

import requests

logger = logging.getLogger(__name__)

ENDPOINT_RE = re.compile(r'Endpoint\s*=\s*(?P<ip>\d+\.\d+\.\d+\.\d+)')
IPINFO_URL = 'https://ipinfo.io'


def determine_ip_location(ip: str) -> Dict:
    """Determine location of IP address.

    Args:
        ip: IP address
    Returns:
        dict containing (among others) fields country, region, city
    """
    req = requests.get(f'{IPINFO_URL}/{ip}')
    return req.json()


def find_vpn_server_locations(wg_config_files: List[str]):
    """Find geographic locations of wireguard VPN server endpoints.

    Args:
        wg_config_files: list of wireguard config files/directories
    Returns:
        list of dictionaries containing configuration/location data.
    """
    wg_configs = []
    for conf_path in wg_config_files:
        p_conf_path = pathlib.Path(conf_path)
        if p_conf_path.is_dir():
            # Find all wireguard config files in given directory:
            wg_configs.extend(
                {'conf': gp for gp in p_conf_path.glob('*.conf')}
            )
        else:
            # The file itself is assumed to be a wireguard config file:
            wg_configs.append({'conf': p_conf_path})

    # Extract IP addresses of server endpoints from wireguard config files
    # and determine their geographic location:
    for wg_config in wg_configs:
        conf = wg_config['conf'].read_text()
        match = ENDPOINT_RE.search(conf)
        if match:
            ip = match.group('ip')
            wg_config.update(determine_ip_location(ip))
        else:
            wg_config['ip'] = None
            logger.warning(
                'Cannot find endpoint IP in wireguard configfile %s', wg_config['conf']
            )
    return wg_configs



"""Console script for mozvpn."""
import sys
import csv
import click

from mozvpn import mozvpn, mozvpn_gui


@click.group()
def main():
    ...


@main.command()
def gui():
    mozvpn_gui.gui()


@click.argument('config_paths', nargs=-1)
@click.option('-o', '--output', type=click.File('w'), default='-')
@main.command()
def geolocate(config_paths, output):
    """Determine geographic location for server endpoints in given config files or directories.

    Write the csv-formatted vpn server information to output file, or to stdout,
    if 'output' was not provided or is a dash ('-').
    """
    vpn_configs = mozvpn.find_vpn_server_locations(config_paths)
    fieldnames = ['conf', 'ip', 'country', 'region', 'city']
    writer = csv.DictWriter(output, fieldnames, extrasaction='ignore')
    writer.writeheader()
    writer.writerows(vpn_configs)


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover
